@@ -5,6 +5,8 @@
 pub mod crypto;
 /// All functions and structs related to error handling
 pub(crate) mod error;
+/// All git related operations.
+pub mod git;
 /// This is the library part of ripasso, it implements the functions needed to manipulate a pass
 /// directory.
 pub mod pass;
