@@ -4,7 +4,7 @@ use crate::{error::Result, git::should_sign, test_helpers::UnpackedDir};
 fn test_should_sign_true() -> Result<()> {
     let dir = UnpackedDir::new("test_should_sign_true")?;
 
-    let repo = git2::Repository::open(dir.dir()).unwrap();
+    let repo = git2::Repository::open(dir.path())?;
 
     let result = should_sign(&repo);
 
@@ -17,7 +17,7 @@ fn test_should_sign_true() -> Result<()> {
 fn test_should_sign_false() -> Result<()> {
     let dir = UnpackedDir::new("test_should_sign_false")?;
 
-    let repo = git2::Repository::open(dir.dir()).unwrap();
+    let repo = git2::Repository::open(dir.path())?;
 
     let result = should_sign(&repo);
 
